from decimal import Decimal

from django.db.models import DecimalField

from netbox import settings
from netbox.views.generic import ObjectView, ObjectListView, ObjectEditView, ObjectDeleteView, BulkImportView, \
    BulkEditView, BulkDeleteView
from ocp_project_plugin.filters import OCPProjectFilter
from ocp_project_plugin.forms import (
    OCPProjectImportForm,
    OCPProjectFilterForm,
    OCPProjectForm,
    OCPProjectBulkEditForm
)
from ocp_project_plugin.models import OCPProject, AppEnvironment
from ocp_project_plugin.tables import OCPProjectTable, AppEnvironmentTable
from utilities.views import register_model_view

PLUGIN_SETTINGS = settings.PLUGINS_CONFIG.get('ocp_project_plugin', dict())
CPU_COST = PLUGIN_SETTINGS.get('cpu_cost', '')
MEMORY_COST = PLUGIN_SETTINGS.get('memory_cost', '')
STORAGE_COST = PLUGIN_SETTINGS.get('storage_cost', '')


class OCPProjectListView(ObjectListView):
    queryset = OCPProject.objects.all()
    filterset = OCPProjectFilter
    filterset_form = OCPProjectFilterForm
    table = OCPProjectTable


class OCPProjectEditView(ObjectEditView):
    """View for editing OCP Project instance."""

    queryset = OCPProject.objects.all()
    form = OCPProjectForm
    default_return_url = "plugins:ocp_project_plugin:ocpproject_list"


class OCPProjectDeleteView(ObjectDeleteView):
    queryset = OCPProject.objects.all()
    default_return_url = "plugins:ocp_project_plugin:ocpproject_list"


class OCPProjectBulkImportView(BulkImportView):
    queryset = OCPProject.objects.all()
    model_form = OCPProjectImportForm
    table = OCPProjectTable
    default_return_url = "plugins:ocp_project_plugin:ocpproject_list"


class OCPProjectBulkEditView(BulkEditView):
    queryset = OCPProject.objects.all()
    filterset = OCPProjectFilter
    table = OCPProjectTable
    form = OCPProjectBulkEditForm


class OCPProjectBulkDeleteView(BulkDeleteView):
    queryset = OCPProject.objects.all()
    table = OCPProjectTable


@register_model_view(OCPProject)
class OCPProjectView(ObjectView):
    template_name = 'ocp_project_plugin/ocp_project/ocp_project.html'
    queryset = OCPProject.objects.all()

    def get_extra_context(self, request, instance):
        app_environment_assignments = AppEnvironment.objects.restrict(request.user, 'view').filter(
            ocp_project=instance
        )
        assignments_table = AppEnvironmentTable(app_environment_assignments, user=request.user)
        assignments_table.configure(request)

        total_cpu = 0
        total_memory = 0
        total_storage = 0
        total_cpu_cost = 0
        total_memory_cost = 0
        total_storage_cost = 0

        for app_env in app_environment_assignments:
            if app_env.limits_cpu is not None:
                total_cpu = + DecimalField().to_python(app_env.limits_cpu)

        for app_env in app_environment_assignments:
            total_memory = + app_env.get_limits_memory_gi()
            # total_cpu_cost = + app_env.calculate_cpu_cost()
            # print(app_env.calculate_memory_cost())

        total_cpu_cost = Decimal(CPU_COST) * Decimal(total_cpu)
        total_memory_cost = Decimal(MEMORY_COST) * Decimal(total_memory)
        total_cost = total_cpu_cost + total_memory_cost + total_storage_cost

        return {
            'assignments_table': assignments_table,
            'assignment_count': AppEnvironment.objects.filter(ocp_project=instance).count(),
            'app_env_list': AppEnvironment.objects.filter(ocp_project=instance),
            'total_cpu': total_cpu,
            'total_memory': total_memory,
            'total_storage': total_storage,
            'total_cpu_cost': total_cpu_cost,
            'total_memory_cost': total_memory_cost,
            'total_storage_cost': total_storage_cost,
            'total_cost': total_cost
        }



from extras.plugins import PluginConfig

__version__ = "0.0.0.0.242"


class OCPProjectConfig(PluginConfig):
    name = "ocp_project_plugin"
    verbose_name = "OCP Project Plugin"
    description = "The netbox ocp project plugin, for creating ocp projects"
    min_version = "3.4.0"
    version = __version__
    author = "Tim Rhomberg"
    author_email = "timrhomberg@hotmail.com"
    required_settings = [
        "gitlab_project_url",
        "values_path",
        "default_access_token",
        'jira_browse_url',
        'ocp_tst_url',
        'ocp_dev_url',
        'ocp_int_url',
        'ocp_prd_url',
        'argocd_tst_url',
        'argocd_dev_url',
        'argocd_int_url',
        'argocd_prd_url',
        'prometheus_tst_url',
        'prometheus_dev_url',
        'prometheus_int_url',
        'prometheus_prd_url',
        'grafana_tst_url',
        'grafana_dev_url',
        'grafana_int_url',
        'grafana_prd_url',
        'kibana_tst_url',
        'kibana_dev_url',
        'kibana_int_url',
        'kibana_prd_url',
        'cpu_cost',
        'memory_cost',
        'storage_cost'
    ]
    base_url = "ocp-project-plugin"


config = OCPProjectConfig



from rest_framework import serializers

from netbox.api.serializers import WritableNestedSerializer
from ocp_project_plugin.models import AppEnvironment, OCPProject


#
# App Environment
#

class NestedAppEnvironmentSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:ocp_project_plugin-api:appenvironment-detail"
    )

    class Meta:
        model = AppEnvironment
        fields = ["id", "url", "display", "app_env"]


#
# OCP Project
#
class NestedOCPProjectSerializer(WritableNestedSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name="plugins-api:ocp_project_plugin-api:ocpproject-detail"
    )

    class Meta:
        model = OCPProject
        fields = ["id", "url", "display", "name"]



from django import forms
from django.db import models
from django.core import exceptions, validators


class MemoryResourceFormField(forms.Field):
    def __init__(
        self, *, empty_value="", **kwargs
    ):
        self.empty_value = empty_value
        super().__init__(**kwargs)
        self.validators.append(validators.ProhibitNullCharactersValidator())

    def to_python(self, value):
        """Return a string."""
        if value in self.empty_values:
            return self.empty_value

        if str(value).endswith('Mi'):
            string_value = str(value)[:-2]
            int_value = int(string_value) / 1000
            return f"{int_value}Gi"

        # Return Gigabyte
        if str(value).endswith('Gi'):
            return f"{value}"

        return value


class MemoryResourceField(models.Field):
    empty_strings_allowed = False
    default_error_messages = {
        "invalid": "“%(value)s” value must end with Mi (Megabyte) or Gi (Gigabyte)",
        "invalid_nullable": "“%(value)s” value must contain a value"
    }
    description = "Memory resource field, suffix allowed: Mi, Gi"

    def get_internal_type(self):
        return "MemoryResourceField"

    def to_python(self, value):
        # Null or empty
        if self.null and value in self.empty_values:
            return None

        # Megabyte to Gigabyte
        if str(value).endswith('Mi'):
            string_value = str(value)[:-2]
            int_value = int(string_value) / 1000
            return f"{int_value}Gi"

        # Return Gigabyte
        if str(value).endswith('Gi'):
            return f"{value}"

        raise exceptions.ValidationError(
            self.error_messages["invalid_nullable" if self.null else "invalid"],
            code="invalid",
            params={"value": value},
        )

    def get_prep_value(self, value):
        value = super().get_prep_value(value)
        if value is None:
            return None
        return self.to_python(value)

    def formfield(self, **kwargs):
        form_class = MemoryResourceFormField
        defaults = {"form_class": form_class, "required": False}
        return super().formfield(**{**defaults, **kwargs})



from django.forms import (
    CharField,
    BooleanField, PasswordInput, ModelChoiceField,
)

from ipam.models import Prefix, IPAddress
from netbox.forms import (
    NetBoxModelBulkEditForm,
    NetBoxModelFilterSetForm,
    NetBoxModelImportForm,
    NetBoxModelForm,
)
from ocp_project_plugin.models import AppEnvironment, OCPProject
from utilities.forms import DynamicModelChoiceField, APISelectMultiple, CSVModelChoiceField


class AppEnvironmentAddForm(NetBoxModelForm):
    """Form for creating a new App Environment object."""
    ocp_project = DynamicModelChoiceField(
        queryset=OCPProject.objects.all(),
        label="OCP Project",
        help_text="Choose the ocp project e.g. web shop",
    )
    egress_ip = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(),
        #queryset=Prefix.get_child_ips(Prefix.objects.get(prefix='10.190.1.64/26')),
        #queryset=IPAddress.objects.all(),
        label="Egress IP",
        help_text="Select your egress ip",
        widget=APISelectMultiple(
            api_url='/api/ipam/ip-addresses/?q=&parent=10.190.1.64/26',
        ),
        required=False
    )

    fieldsets = (
        ('OCP Project', ['ocp_project']),
        ('Environment', ('cluster_env', 'app_env')),
        ('Deployment', ('repo', 'branch', 'access_token', 'path', 'deployment_kind')),
        ('Monitoring', ('monitoring', 'postgres_monitoring')),
        ('Resources', ('limits_cpu', 'limits_memory', 'requests_cpu', 'requests_memory')),
        ('Additional Config', ('mtls', 'egress_ip')),
    )

    class Meta:
        model = AppEnvironment

        fields = ["access_token", "cluster_env", "app_env", "mtls", "repo", "branch", "path", "egress_ip", "deployment_kind", "monitoring",
                  "postgres_monitoring", "ocp_project", "requests_cpu", "requests_memory", "limits_cpu",
                  "limits_memory"]

        widgets = {
            'access_token': PasswordInput(),
        }


class AppEnvironmentEditForm(NetBoxModelForm):
    """Form for creating a new App Environment object."""
    ocp_project = DynamicModelChoiceField(
        queryset=OCPProject.objects.all(),
        label="OCP Project",
        help_text="Choose the ocp project e.g. web shop",
        disabled=True
    )
    egress_ip = DynamicModelChoiceField(
        queryset=IPAddress.objects.all(),
        label="Egress IP",
        help_text="Select your egress ip",
        widget=APISelectMultiple(
            api_url='/api/ipam/ip-addresses/?q=&parent=10.190.1.64/26',
        ),
        required = False
    )

    fieldsets = (
        ('OCP Project', ['ocp_project']),
        ('Environment', ('cluster_env', 'app_env')),
        ('Deployment', ('repo', 'branch', 'access_token', 'path', 'deployment_kind')),
        ('Monitoring', ('monitoring', 'postgres_monitoring')),
        ('Resources', ('limits_cpu', 'limits_memory', 'requests_cpu', 'requests_memory')),
        ('Additional Config', ('mtls', 'egress_ip')),
    )

    class Meta:
        model = AppEnvironment

        fields = ["access_token", "cluster_env", "app_env", "mtls", "repo", "branch", "path", "egress_ip", "deployment_kind", "monitoring",
                  "postgres_monitoring", "ocp_project", "requests_cpu", "requests_memory", "limits_cpu",
                  "limits_memory"]

        widgets = {
            'access_token': PasswordInput(),
        }


class AppEnvironmentFilterForm(NetBoxModelFilterSetForm):
    """Form for filtering App Environment instances."""

    model = AppEnvironment

    app_env = CharField(
        required=False,
        label="App Env",
        help_text="The app Env String used for creating the namespace e.g. tst",
    )
    mtls = BooleanField(
        required=False,
        label="MTLS",
        help_text="Enable if mtls should be used",
    )
    repo = CharField(
        required=False,
        label="Git Repository",
        help_text="Path of git Repository, don't forget the .git at the end e.g. "
                  "https://gitlab.com/example/example-deployment-manifests.git",
    )
    branch = CharField(
        required=False,
        label="Git Branch",
        help_text="The git Branch of the Repository e.g. main"
    )
    path = CharField(
        required=False,
        label="Git Path",
        help_text="Path of the deployment files e.g. overlays/tst"
    )
    egress_ip = CharField(
        required=False,
        label="Egress IP",
        help_text="The egress IP e.g. 10.10.10.10"
    )
    monitoring = BooleanField(
        required=False,
        label="Monitoring",
        help_text="Enable if monitoring should be used",
    )
    postgres_monitoring = BooleanField(
        required=False,
        label="Postgres Monitoring",
        help_text="Enable if postgres monitoring should be used",
    )


class AppEnvironmentImportForm(NetBoxModelImportForm):
    cluster_env = CharField(
        label="App Env",
        help_text="The app Env String used for creating the namespace e.g. tst",
    )
    app_env = CharField(
        label="App Env",
        help_text="The app Env String used for creating the namespace e.g. tst",
    )
    mtls = BooleanField(
        required=False,
        label="MTLS",
        help_text="Enable if mtls should be used",
    )
    repo = CharField(
        label="Git Repository",
        help_text="Path of git Repository, don't forget the .git at the end e.g. "
                  "https://gitlab.com/example/example-deployment-manifests.git",
    )
    branch = CharField(
        label="Git Branch",
        help_text="The git Branch of the Repository e.g. main"
    )
    path = CharField(
        label="Git Path",
        help_text="Path of the deployment files e.g. overlays/tst"
    )
    egress_ip = CharField(
        required=False,
        label="Egress IP",
        help_text="The egress IP e.g. 10.10.10.10"
    )
    monitoring = BooleanField(
        required=False,
        label="Monitoring",
        help_text="Enable if monitoring should be used",
    )
    postgres_monitoring = BooleanField(
        required=False,
        label="Postgres Monitoring",
        help_text="Enable if postgres monitoring should be used",
    )
    ocp_project = CSVModelChoiceField(
        queryset=OCPProject.objects.all(),
        to_field_name='name',
        help_text='Choose the ocpproject'
    )

    class Meta:
        model = AppEnvironment

        fields = ["cluster_env", "app_env", "mtls", "repo", "branch", "path", "egress_ip", "deployment_kind", "monitoring", "postgres_monitoring",
                  "ocp_project"]


class AppEnvironmentBulkEditForm(NetBoxModelBulkEditForm):
    model = AppEnvironment

    app_env = CharField(
        required=False,
        label="App Env",
        help_text="The app Env String used for creating the namespace e.g. tst",
    )
    mtls = BooleanField(
        required=False,
        label="MTLS",
        help_text="Enable if mtls should be used",
    )
    repo = CharField(
        required=False,
        label="Git Repository",
        help_text="Path of git Repository, don't forget the .git at the end e.g. "
                  "https://gitlab.com/example/example-deployment-manifests.git",
    )
    branch = CharField(
        required=False,
        label="Git Branch",
        help_text="The git Branch of the Repository e.g. main"
    )
    path = CharField(
        required=False,
        label="Git Path",
        help_text="Path of the deployment files e.g. overlays/tst"
    )
    egress_ip = CharField(
        required=False,
        label="Egress IP",
        help_text="The egress IP e.g. 10.10.10.10"
    )
    monitoring = BooleanField(
        required=False,
        label="Monitoring",
        help_text="Enable if monitoring should be used",
    )
    postgres_monitoring = BooleanField(
        required=False,
        label="Postgres Monitoring",
        help_text="Enable if postgres monitoring should be used",
    )



from ocp_project_plugin.models import AppEnvironment, OCPProject
from netbox.search import SearchIndex, register_search


@register_search
class OCPProjectIndex(SearchIndex):
    model = OCPProject
    fields = (
        ('name', 100),
        ('description', 100),
        ('display_name', 100),
        ('project_owner', 100),
        ('customer', 100),
        ('docu_url', 100),
        ('workload', 100),
        ('request', 100),
    )


@register_search
class AppEnvironmentIndex(SearchIndex):
    model = AppEnvironment
    fields = (
        ('cluster_env', 100),
        ('app_env', 100),
        ('mtls', 100),
        ('repo', 100),
        ('branch', 100),
        ('path', 100),
        ('egress_ip', 100),
        ('deployment_kind', 100),
        ('monitoring', 100),
        ('postgres_monitoring', 100),
        ('ocp_project', 100),
        ('requests_cpu', 100),
        ('requests_memory', 100),
        ('limits_cpu', 100),
        ('limits_memory', 100),
    )



import yaml
from django.conf import settings
from django_rq import job
from git import Repo

PLUGIN_SETTINGS = settings.PLUGINS_CONFIG.get('ocp_project_plugin', dict())
GITLAB_PROJECT_URL = PLUGIN_SETTINGS.get('gitlab_project_url', '')
VALUES_PATH = PLUGIN_SETTINGS.get('jira_browse_url', '')

"""
1. Git Repo pullen
2. Überprüfen ob es einen Branch mit dem Ticket Namen schon gibt
3. Neuer Branch erstellen mit dem Ticket Namen
4. Secrets entschlüsseln
5. OCP PRoject/App Environment Model Daten in yaml konvertieren
6. Überprüfen ob OCP Project & App Environment schon existiern in values.yaml
7. YAML Daten dem values.yaml anfügen oder updaten
8. Secrets der Secrets Datei anfügen oder updaten
9. Secret verschlüsseln
10. Mergen
"""


@job("default")
def sync_project(yaml_object):
    print("Step 1: Print new yaml Object")
    print(yaml_object)
    try:
        print("Step 2: Start cloning git repository")
        repo_instance = Repo.clone_from(GITLAB_PROJECT_URL, '/repo/project-repo')
        print(f"Step 2.1: Cloning finished, {repo_instance}")
    except:
        print("Step: 2: Cloning failed")

    request = yaml_object['request']
    print(f"Step 3: Check if int this repository a branch with the name {request} already exists.")
    repo = Repo('/repo/project-repo')
    remote_refs = repo.remote().refs

    found = False
    for refs in remote_refs:
        if refs.name == 'origin/' + request:
            found = True
        # print(refs.name)

    if found:
        print("Step 4: Branch already exists")
    else:
        print("Step 4: Branch doesn't exists")
        branch_name = request
        current = repo.create_head(branch_name)
        current.checkout()

        print(f"Step 4.1: Create branch {branch_name}")

        repo.git.push('--set-upstream', 'origin', current)

        print(f"Step 4.2: Branch {branch_name} pushed")

    print("Step 5: Read project values file")
    with open('/repo/project-repo/cluster/projects/values.yaml', 'r') as file:
        cur_yaml = yaml.safe_load(file)  # Note the safe_load

    index = -1
    for idx, project in enumerate(cur_yaml['projects']):
        if project['name'] == yaml_object['name']:
            print(f"Step 5.1: Project already exists, Index Key: {idx}")
            index = idx

    print("Step 6: Sync project values")
    if index > -1:
        cur_yaml['projects'][index] = yaml_object
    else:
        cur_yaml['projects'].append(yaml_object)

    print("Step 7: Save project values")
    if cur_yaml:
        with open('/repo/project-repo/cluster/projects/values.yaml', 'w') as file:
            yaml.safe_dump(cur_yaml, file, sort_keys=False)

    changedFiles = [item.a_path for item in repo.index.diff(None)]
    if len(changedFiles) > 0:
        repo.config_writer().set_value("user", "name", "myusername").release()
        repo.config_writer().set_value("user", "email", "myemail").release()
        repo.git.add('*')
        repo.git.commit(m="Sync OCP Project")
        repo.git.push()

    print("Step 8: Changes pushed in the repository")

    repo = Repo('/repo/project-repo')
    repo.git.checkout('main')
    repo.git.merge(request)
    repo.git.push()
    print("Step 9: Changes merged in the main branch")

    remote = repo.remote(name='origin')
    remote.push(refspec=f":{request}")
    print("Step 10: branch deleted remote")
    repo.delete_head(request)
    print("Step 11: branch deleted local")



from django import template
from django.conf import settings

from ipam.models import Prefix, IPAddress
from ocp_project_plugin.models import AppEnvironment

register = template.Library()

PLUGIN_SETTINGS = settings.PLUGINS_CONFIG.get('ocp_project_plugin', dict())
GITLAB_PROJECT_URL = PLUGIN_SETTINGS.get('gitlab_project_url', '')
JIRA_BROWSE_URL = PLUGIN_SETTINGS.get('jira_browse_url', '')
CPU_COST = PLUGIN_SETTINGS.get('cpu_cost', '')
MEMORY_COST = PLUGIN_SETTINGS.get('memory_cost', '')
STORAGE_COST = PLUGIN_SETTINGS.get('storage_cost', '')
OCP_TST_URL = PLUGIN_SETTINGS.get('ocp_tst_url', '')
OCP_DEV_URL = PLUGIN_SETTINGS.get('ocp_dev_url', '')
OCP_INT_URL = PLUGIN_SETTINGS.get('ocp_int_url', '')
OCP_PRD_URL = PLUGIN_SETTINGS.get('ocp_prd_url', '')
ARGOCD_TST_URL = PLUGIN_SETTINGS.get('argocd_tst_url', '')
ARGOCD_DEV_URL = PLUGIN_SETTINGS.get('argocd_dev_url', '')
ARGOCD_INT_URL = PLUGIN_SETTINGS.get('argocd_int_url', '')
ARGOCD_PRD_URL = PLUGIN_SETTINGS.get('argocd_prd_url', '')
PROMETHEUS_TST_URL = PLUGIN_SETTINGS.get('prometheus_tst_url', '')
PROMETHEUS_DEV_URL = PLUGIN_SETTINGS.get('prometheus_dev_url', '')
PROMETHEUS_INT_URL = PLUGIN_SETTINGS.get('prometheus_int_url', '')
PROMETHEUS_PRD_URL = PLUGIN_SETTINGS.get('prometheus_prd_url', '')
GRAFANA_TST_URL = PLUGIN_SETTINGS.get('grafana_tst_url', '')
GRAFANA_DEV_URL = PLUGIN_SETTINGS.get('grafana_dev_url', '')
GRAFANA_INT_URL = PLUGIN_SETTINGS.get('grafana_int_url', '')
GRAFANA_PRD_URL = PLUGIN_SETTINGS.get('grafana_prd_url', '')
KIBANA_TST_URL = PLUGIN_SETTINGS.get('kibana_tst_url', '')
KIBANA_DEV_URL = PLUGIN_SETTINGS.get('kibana_dev_url', '')
KIBANA_INT_URL = PLUGIN_SETTINGS.get('kibana_int_url', '')
KIBANA_PRD_URL = PLUGIN_SETTINGS.get('kibana_prd_url', '')


# settings value
@register.simple_tag
def jira_browse_url(ticket_id):
    return f"{JIRA_BROWSE_URL}{ticket_id}"


@register.simple_tag
def get_cpu_cost():
    return CPU_COST


@register.simple_tag
def calculate_cpu_cost(amount):
    if amount is '':
        return '-'
    else:
        return int(CPU_COST) * int(amount)


@register.simple_tag
def calculate_memory_cost(amount):
    if amount is '':
        return '-'
    else:
        if amount.endswith('Mi'):
            return int(MEMORY_COST) * float(str(amount)[:-2]) / 1000
        else:
            return int(MEMORY_COST) * float(str(amount)[:-2])


@register.simple_tag
def get_memory_cost():
    return MEMORY_COST


@register.simple_tag
def calculate_storage_cost(amount):
    if amount is '':
        return '0'
    else:
        return int(STORAGE_COST) * int(amount)


@register.simple_tag
def get_storage_cost():
    return STORAGE_COST


@register.simple_tag
def calculate_total_cost(cpu_amount, memory_amount, storage_amount):
    print(cpu_amount)
    print(memory_amount)
    if cpu_amount is '' or memory_amount is '':
        return '0'
    else:
        return calculate_cpu_cost(cpu_amount) + calculate_memory_cost(memory_amount) + 0


@register.simple_tag
def get_ocp_resource_quota_url(cluster_env, app_env, ocp_project_name):
    suffix = '/k8s/ns/' + ocp_project_name + '-' + app_env + '/resourcequotas'
    if cluster_env == 'TST':
        return OCP_TST_URL + suffix
    if cluster_env == 'DEV':
        return OCP_DEV_URL + suffix
    if cluster_env == 'INT':
        return OCP_INT_URL + suffix
    if cluster_env == 'PRD':
        return OCP_PRD_URL + suffix


@register.simple_tag
def get_url(app, cluster_env):
    print("1")
    print(Prefix.objects.get(prefix='10.190.1.64/26'))
    print("2")
    print(Prefix.get_child_ips(Prefix.objects.get(prefix='10.190.1.64/26')))
    print("3")
    print(IPAddress.objects.filter(address__net_host_contained='10.190.1.64/26'))
    print("4")
    if cluster_env == 'TST':
        if app == 'argocd':
            return ARGOCD_TST_URL
        if app == 'grafana':
            return GRAFANA_TST_URL
        if app == 'ocp':
            return OCP_TST_URL
        if app == 'prometheus':
            return PROMETHEUS_TST_URL
        if app == 'kibana':
            return KIBANA_TST_URL
    if cluster_env == 'DEV':
        if app == 'argocd':
            return ARGOCD_DEV_URL
        if app == 'grafana':
            return GRAFANA_DEV_URL
        if app == 'ocp':
            return OCP_DEV_URL
        if app == 'prometheus':
            return PROMETHEUS_DEV_URL
        if app == 'kibana':
            return KIBANA_DEV_URL
    if cluster_env == 'INT':
        if app == 'argocd':
            return ARGOCD_DEV_URL
        if app == 'grafana':
            return GRAFANA_DEV_URL
        if app == 'ocp':
            return OCP_DEV_URL
        if app == 'prometheus':
            return PROMETHEUS_DEV_URL
        if app == 'kibana':
            return KIBANA_DEV_URL
    if cluster_env == 'PRD':
        if app == 'argocd':
            return ARGOCD_DEV_URL
        if app == 'grafana':
            return GRAFANA_DEV_URL
        if app == 'ocp':
            return OCP_DEV_URL
        if app == 'prometheus':
            return PROMETHEUS_DEV_URL
        if app == 'kibana':
            return KIBANA_DEV_URL


@register.simple_tag
def count_app_environments(object_id):
    return AppEnvironment.objects.filter(ocp_project=object_id).count()


@register.simple_tag
def get_all_app_environments(object_id):
    return AppEnvironment.objects.filter(ocp_project=object_id)



from utilities.choices import ChoiceSet


class AppEnvironmentClusterEnvChoices(ChoiceSet):
    key = 'AppEnvironment.cluster_env'

    CHOICE_PRD = 'PRD'
    CHOICE_INT = 'INT'
    CHOICE_TST = 'TST'
    CHOICE_DEV = 'DEV'

    CHOICES = [
        (CHOICE_PRD, 'PRD', 'red'),
        (CHOICE_INT, 'INT', 'yellow'),
        (CHOICE_TST, 'TST', 'cyan'),
        (CHOICE_DEV, 'DEV', 'light green'),
    ]


class AppEnvironmentDeploymentKindChoices(ChoiceSet):
    key = 'AppEnvironment.deployment_kind'

    DEPLOYMENT_KIND_HELM = 'HELM'
    DEPLOYMENT_KIND_KUSTOMIZE = 'KUSTOMIZE'
    DEPLOYMENT_KIND_NORMAL = 'NORMAL'

    CHOICES = [
        (DEPLOYMENT_KIND_HELM, 'HELM', 'gray'),
        (DEPLOYMENT_KIND_KUSTOMIZE, 'KUSTOMIZE', 'green'),
        (DEPLOYMENT_KIND_NORMAL, 'NORMAL', 'cyan'),
    ]



from netbox.api.routers import NetBoxRouter
from ocp_project_plugin.api.views import OCPProjectPluginRootView, AppEnvironmentViewSet, OCPProjectViewSet

router = NetBoxRouter()
router.APIRootView = OCPProjectPluginRootView

router.register("appenvironment", AppEnvironmentViewSet)
router.register("ocpproject", OCPProjectViewSet)

urlpatterns = router.urls



from django.urls import path

from netbox.views.generic import ObjectChangeLogView

from ocp_project_plugin.models import AppEnvironment, OCPProject
from ocp_project_plugin.views import (
    # App Environment Views
    AppEnvironmentListView,
    AppEnvironmentView,
    AppEnvironmentEditView,
    AppEnvironmentDeleteView,
    AppEnvironmentBulkImportView,
    AppEnvironmentBulkEditView,
    AppEnvironmentBulkDeleteView,
    # OCP Project Views
    OCPProjectListView,
    OCPProjectView,
    OCPProjectEditView,
    OCPProjectDeleteView,
    OCPProjectBulkImportView,
    OCPProjectBulkEditView,
    OCPProjectBulkDeleteView, AppEnvironmentAddView,
)

app_name = "ocp_project_plugin"

urlpatterns = [
    #
    # App Environment urls
    #
    path("appenvironment/", AppEnvironmentListView.as_view(), name="appenvironment_list"),
    path("appenvironment/add/", AppEnvironmentAddView.as_view(), name="appenvironment_add"),
    path("appenvironment/import/", AppEnvironmentBulkImportView.as_view(), name="appenvironment_import"),
    path("appenvironment/edit/", AppEnvironmentBulkEditView.as_view(), name="appenvironment_bulk_edit"),
    path("appenvironment/delete/", AppEnvironmentBulkDeleteView.as_view(), name="appenvironment_bulk_delete"),
    path("appenvironment/<int:pk>/", AppEnvironmentView.as_view(), name="appenvironment"),
    path("appenvironment/<int:pk>/edit/", AppEnvironmentEditView.as_view(), name="appenvironment_edit"),
    path("appenvironment/<int:pk>/delete/", AppEnvironmentDeleteView.as_view(), name="appenvironment_delete"),
    path(
        "appenvironment/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="appenvironment_changelog",
        kwargs={"model": AppEnvironment},
    ),
    #
    # OCP Project urls
    #
    path("ocpproject/", OCPProjectListView.as_view(), name="ocpproject_list"),
    path("ocpproject/add/", OCPProjectEditView.as_view(), name="ocpproject_add"),
    path("ocpproject/import/", OCPProjectBulkImportView.as_view(), name="ocpproject_import"),
    path("ocpproject/edit/", OCPProjectBulkEditView.as_view(), name="ocpproject_bulk_edit"),
    path("ocpproject/delete/", OCPProjectBulkDeleteView.as_view(), name="ocpproject_bulk_delete"),
    path("ocpproject/<int:pk>/", OCPProjectView.as_view(), name="ocpproject"),
    path("ocpproject/<int:pk>/edit/", OCPProjectEditView.as_view(), name="ocpproject_edit"),
    path("ocpproject/<int:pk>/delete/", OCPProjectDeleteView.as_view(), name="ocpproject_delete"),
    path(
        "ocpproject/<int:pk>/changelog/",
        ObjectChangeLogView.as_view(),
        name="ocpproject_changelog",
        kwargs={"model": OCPProject},
    ),
]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ocp_project_plugin',
 'ocp_project_plugin.api',
 'ocp_project_plugin.fields',
 'ocp_project_plugin.filters',
 'ocp_project_plugin.forms',
 'ocp_project_plugin.migrations',
 'ocp_project_plugin.models',
 'ocp_project_plugin.tables',
 'ocp_project_plugin.templatetags',
 'ocp_project_plugin.views']

package_data = \
{'': ['*'],
 'ocp_project_plugin': ['templates/ocp_project_plugin/*',
                        'templates/ocp_project_plugin/app_environment/*',
                        'templates/ocp_project_plugin/ocp_project/*']}

setup_kwargs = {
    'name': 'ocp-project-plugin',
    'version': '0.0.0.0.242',
    'description': 'Netbox OCP Project Plugin',
    'long_description': '# General\n## Build Project\nTo build the project go to login in the pypi web ui and get your token. Add your token to the local pypi config.\n```\npoetry config pypi-token.pypi pypi-\n```\nAfter you made changes, change the version in the files pyproject.toml and netbox_storage/__init__.py\n\nNow you can build and publish the project.\n```\npoetry publish --build\n```\n\n## Use Project\nLink: https://github.com/netbox-community/netbox-docker/wiki/Using-Netbox-Plugins\n\ndocker-compose build --no-cache && docker-compose build --no-cache && docker-compose up -d\n\n\n## Directory structure\n\n```\n+- api - The API Classes, consitsts of Serializer, URL Mapper and Views\n+- filters - Filters of the models, the implementation of the method search, for searching\n+- forms - The ModelForm, ModelFilterForm, ModelImportForm, ModelBulkEditForm, the forms which will be displayed\n+- migrations - DB Django Migration steps\n+- tables - The ModelTable, which has the configuration on how the table looks like\n+- templates\n  +- netbox_storage - The detail view of each model\n    +- drive - The template content of drive, with base and partition model\n    +- inc - The template content box in the Virtual Machine Model\n    +- partition - The template content of partition, with base and physicalvolume model\n    +- physicalvolume - The template content of physicalvolume with base and linuxvolume model\n    +- volumegroup - The template content of volumegroup with base, logicalvolume and physicalvolume\n+- views - PhysicalvolumeListView, PhysicalvolumeView, PhysicalvolumeEditView, PhysicalvolumeDeleteView, \n           PhysicalvolumeBulkImportView, PhysicalvolumeBulkEditView, PhysicalvolumeBulkDeleteView\n```\n## ERM\n\n![The ERM of the Project](documents/erm.jpg?raw=true "ERM Diagram")\n\n## Queues / Worker\n\n### 1. Job - add_project\n1. Git Repo pullen\n2. Überprüfen ob es einen Branch mit dem Ticket Namen schon gibt\n3. Neuer Branch erstellen mit dem Ticket Namen\n4. Secrets entschlüsseln\n5. OCPPRoject/AppEnvironment Model Daten in yaml konvertieren\n6. YAML Daten dem values.yaml anfügen\n7. Secrets der Secrets Datei anfügen\n8. Secret verschlüsseln\n9. Mergen\n\n',
    'author': 'Tim Rhomberg',
    'author_email': 'timrhomberg@hotmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.db.models import Q

from netbox.filtersets import NetBoxModelFilterSet

from ocp_project_plugin.models import OCPProject


class OCPProjectFilter(NetBoxModelFilterSet):
    class Meta:
        model = OCPProject
        fields = ['name', 'description', 'display_name', 'project_owner', 'customer', 'docu_url', 'workload',
                  'request']

    def search(self, queryset, name, value):
        """Perform the filtered search."""
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name__icontains=value)
            | Q(description__icontains=value)
            | Q(display_name__icontains=value)
            | Q(docu_url__icontains=value)
            | Q(workload__icontains=value)
            | Q(request__icontains=value)
        )
        return queryset.filter(qs_filter)
